// @ts-nocheck
require("dotenv").config({ path: ".env.platform" });
import { Before, After, BeforeAll, AfterAll, Status, setDefaultTimeout, AfterStep } from "@cucumber/cucumber";
import { chromium, firefox, webkit, Page, Browser, BrowserContext, APIRequestContext, request } from "@playwright/test";
import { config } from "./config";
import { baseUrl } from "./urls-config";
import { browserName } from "./browser-config";
import AzureIntegration from "../src/helper/azure-integration";
import Action from "../src/helper/actions";


require("dotenv").config("../platform");

setDefaultTimeout(60 * 1000);

let page: Page;
let browser: Browser;
let context: Promise<BrowserContext>;
let latestScreenShot: string;

BeforeAll(async () => {
  Action.setEnvVariableIfNotExists("BROWSER", process.env.BROWSER)

  switch (browserName[process.env.BROWSER]) {
    case "firefox":
      browser = await firefox.launch(config.browserOptions);
      break;
    case "webkit":
      browser = await webkit.launch(config.browserOptions);
      break;
    case "chromium":
      browser = await chromium.launch(config.browserOptions);
      // browser = await webkit.launch(config.browserOptions);
      // browser = await firefox.launch(config.browserOptions);
      break;
    default:
      browser = await webkit.launch(config.browserOptions);
      browser = await firefox.launch(config.browserOptions);
      browser = await chromium.launch(config.browserOptions);
  }

  try {
    context = browser.newContext({ /* pass any options */ ignoreHTTPSErrors: true, });
    page = await (await context).newPage();
    console.log('\x1b[34m%s\x1b[0m', 'INFO: ' + 'Succesfully launched ' + process.env.BROWSER + ' browser');
  } catch (error) {
    console.log(`An error occured while trying to launch the browser. ${error}`);
  }
  await page.setViewportSize({
    width: 1500,
    height: 920,
  });

  await page.goto(config.BASEURL);

  return page;
});

After(async function (scenario) {
  const testPlanFilePath = process.env.TEST_PLAN_DATA_FILE;
  if (scenario.result!.status == Status.FAILED) {
    await AzureIntegration.publishTestResults(testPlanFilePath, "Failed");
    var screenshotInBase64 = (await page.screenshot({ path: "screenshots/screenshot.png", fullPage: true })).toString(
      "base64"
    );
   await AzureIntegration.attachScreenShot(screenshotInBase64);
  }
  if (scenario.result!.status == Status.PASSED) {
    // var screenshotInBase64 = (await page.screenshot({ path: "screenshots/screenshot.png", fullPage: true })).toString(
    //   "base64"
    // );
    await AzureIntegration.publishTestResults(testPlanFilePath,"Passed");
  }
});

AfterStep(async function (scenario) {
  if (scenario.result!.status == Status.FAILED) {
    latestScreenShot = (await page.screenshot({ path: "test/test-results/screenshots/screenshot.png", fullPage: true })).toString("base64");
    this.attach(latestScreenShot, 'image/png');
  }
});

AfterAll(async function () {
  await browser.close();
});

export { page, context, browser };
