import { expect } from "@playwright/test";
import { ElementHandle, FrameLocator, Locator, Page } from "playwright";
import { page, context, browser } from "../configs/world";
import * as fs from "fs";
import * as path from "path";
import { rootDirectory } from '../rootDir';

import { readFile } from "fs/promises";

export default class Action {
  readonly page: Page;
  static iframe: FrameLocator;

  constructor(page: Page) {
    this.page = page;
    Action.iframe = page.frameLocator("#iframe");
  }


  public async clickSubmitButton(buttonText: string) {
    await Action.iframe.locator('//span[contains(text(),"' + buttonText + '")]')?.click();
  }

  static selectDropDownOption(option: string) {
    // const dropDownItem = await this.iframe.locator("//button[@role='option' and .//*[contains(text(),'" + option + "')]]").textContent();
    // await locator.selectOption(dropDownItem)
    // await dropDownItem.scrollIntoViewIfNeeded();
    return this.iframe.locator("//button[@role='option' and .//*[contains(text(),'" + option + "')]]")

  }

  static async getRandomDropDownOption() {
    const dropDownOptions = await this.iframe.locator("//button[@role='option']").all();
    const randomIndex = Math.floor(Math.random() * dropDownOptions.length);
    for (const iterator of dropDownOptions) {
      if (iterator.getAttribute("disabled") == null) {
        await iterator.click();
      }
    }
  }



  static getDropDownOption(option: string) {
    return this.iframe.locator("//button[@role='option' and .//*[contains(text(),'" + option + "')]]")
  }

  static async selectApplication(productName: string) {

    // await page.waitForURL("**\/types")
    const productCard = Action.iframe.locator("//h5[contains(text(),'" + productName + "')]");
    if (await productCard.nth(0).isEnabled()) {
      await productCard.nth(0).click();
    }
  }
  public static async clickSubmitButton(buttonText: string) {
    await Action.iframe.locator('//span[contains(text(),"' + buttonText + '")]')?.click();
  }

  static getRandomName(arr: string[]) {
    const randomIndex = Math.floor(Math.random() * arr.length);
    const item = arr[randomIndex];
    return item;
  }

  static async isElementDisplayed(element: Locator) {
    return await element.isVisible({ timeout: 30000 })
  }

  public static generateNumericString(length: number) {
    var alPhanumericString: string = "";
    const characters = "0123456789";
    const charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
      alPhanumericString += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return alPhanumericString;
  }

  public static generateAlphaNumericString(length: number) {
    var alPhanumericString: string = "";
    const characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    const charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
      alPhanumericString += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return alPhanumericString;
  }

  

  static async getCUrrentUrl() {
    await page.waitForLoadState();
    return page.url();
  }

  static async ReadFileContent(filePath: string) {
    let fileStore = `${rootDirectory}/${filePath}`;
    console.log('Root directory: ' + fileStore)
    return await readFile(filePath, "utf8");
  }

  static async WriteToJsonFile(filePath: string, jsonContent: object) {
    let fileStore = `${rootDirectory}/${filePath}`;
    fs.writeFile(fileStore, JSON.stringify(jsonContent, null, 2), (err) => {
      if (err) {
        console.log("An error occured while attempting to read the file", err);
        return
      }
    }
    );
  }

  static async writeOrAppendJSONFile(filename: any, key: string | number, value: any) {
    // var fileStore = path.join(d, `../${filename}`);
    let fileStore = `${rootDirectory}/${filename}`;
    if (fs.existsSync(fileStore)) {
      const data = fs.readFileSync(fileStore, 'utf8');
      let jsonData = JSON.parse(data);
      if (jsonData.hasOwnProperty(key)) {
        jsonData[key].push(value);
      } else {
        jsonData[key] = [value];
      }
      fs.writeFileSync(fileStore, JSON.stringify(jsonData, null, 2));
    } else {
      const jsonData = {
        [key]: [value]
      };
      fs.writeFileSync(fileStore, JSON.stringify(jsonData, null, 2));
    }
  }




  static async uploadDocument(fileName1: string, fileName2?: string, fileName3?: string) {
    try {
      //   await page.evaluate(() => {
      //     let inputElement = document.querySelectorAll('[type="file"]');
      //     inputElement.forEach((ele) => {
      //       ele.removeAttribute("style");
      //     });
      //   });
      await page.setInputFiles("//input[@type='file']", "screenshots/screenshot.png");
    } catch (error) {
      throw new Error(`An error occured while attempting to upload the file : ${error}`);
    }
  }

  static async WriteToFileNow(file: any, data: string | NodeJS.ArrayBufferView) {
    var fileStore = path.join(__dirname, `../${file}`);

    fs.writeFileSync(fileStore, data)

  }



  static async getFirstWordBeforeSpace(text: string): Promise<string | null> {
    const trimmedText = text.trim();
    const indexOfSpace = trimmedText.indexOf(' ');

    if (indexOfSpace === -1) {
      return trimmedText;
    } else {
      return trimmedText.substr(0, indexOfSpace);
    }
  }
 ;
  
  static async  setEnvVariableIfNotExists(variableName: string, value: string) {
    const envFilePath = path.resolve(process.cwd(), '.env.platform');
    let envFileContent = '';
  
    // Check if the .env file exists
    if (fs.existsSync(envFilePath)) {
      // Read the content of the .env file
      envFileContent = fs.readFileSync(envFilePath, { encoding: 'utf-8' });
    }
  
    // Check if the variable already exists in the file
    const variablePattern = new RegExp(`^${variableName}=`, 'm');
    if (!variablePattern.test(envFileContent)) {
      // The variable does not exist, append it to the file
      const newVariable = `${variableName}=${value}\n`;
      fs.appendFileSync(envFilePath, newVariable, { encoding: 'utf-8' });
  
      // Optionally, if you want to set it in the current process as well:
      process.env[variableName] = value;
    }
  }
  
  



}
