// @ts-nocheck
import { ITestCaseHookParameter } from "@cucumber/cucumber";
import { apiContext } from "../../configs/api-config";
import Action from "./actions";
import RestHelper from "./rest-helper";
require("dotenv").config({ path: ".env.platform" });



var testPlanId: string;
var testSuiteId: string;
var testCaseId: string;
var _testPointId: string;
var _testRunId: string;
var testplanData: any;

export default class AzureIntegration {
  static async getTestPlandetails(filePath: string) {
    const data = await Action.ReadFileContent(filePath);
    testplanData = JSON.parse(data);
    try {
      testPlanId = testplanData.testPlanId;
      testSuiteId = testplanData.testSuiteId;
      testCaseId = testplanData.testCaseId;

    } catch (error) {
      console.warn(`\nUnable to retrive the full test plan meta data. Please ensure that you have a valid JSON with test plan id, test suite id, and test case id.`)
    }
  }

  static async getTestPointId(): Promise<string> {
    const values = Object.values(testplanData);
    if (values.length !== 0) {
      const testPoint = await RestHelper.sendGETRequest(
        `/${process.env.ORGANISATION}/${process.env.PROJECT}/_apis/test/plans/${testPlanId}/suites/${testSuiteId}/points?testCaseId=${testCaseId}&api-version=5.0`
      );
      let response;
      try {
        response = await testPoint.json();
        _testPointId = response.value[0].id;

        return _testPointId;
      } catch (error) {
        if (testPoint.status() == 401) {
          console.log('\x1b[33m%s\x1b[0m', 'WARN: ' + 'ADO was unable to authenticate you. Please ensure that your PAT token is valid');
        }
        if (testPoint.status() == 404) {
          console.log('\x1b[33m%s\x1b[0m', 'WARN: ' + `\nUnable to retrieve test point ID. \nPlease make sure that your test plain ID, test suite ID, and test case ID are valid.}`);
          console.warn()
        }
      }
    }
  }

  static async createTestRun(testRunName?: string): Promise<string> {
    if (_testPointId) {
      let restResponse = await apiContext.post(`/${process.env.ORGANISATION}/${process.env.PROJECT}/_apis/test/runs?api-version=5.0`, {
        data: {
          name: "Automated Test" + Date.now(),
          plan: { id: testPlanId },
          pointIds: [_testPointId],
        },
      });
      let response = await restResponse.json();
      _testRunId = response.id;
      return _testRunId;
    }
  }

  static async updateTestResults(outcome: string) {
    if (_testRunId) {
      await apiContext.patch(
        `/${process.env.ORGANISATION}/${process.env.PROJECT}/_apis/test/runs/${_testRunId}/results?api-version=5.1-preview.6`,
        {
          data: [{ id: 100000, outcome: outcome, state: "Completed", comment: "Execution Successful" }],
        }
      );
    }
  }

  static async attachScreenShot(stream: string) {
    await apiContext.post(
      `/${process.env.ORGANISATION}/${process.env.PROJECT}/_apis/test/runs/${_testRunId}/results/100000/attachments?api-version=5.0-preview.1`,
      {
        data: {
          stream: stream,
          fileName: "screenshot.png",
          comment: "Screenshot captured from failed scenario",
          attachmentType: "GeneralAttachment",
        },
      }
    );
  }



  public static async publishTestResults(filePath: string, outcome: string) {


    await AzureIntegration.getTestPlandetails(filePath);
    await AzureIntegration.getTestPointId();
    await AzureIntegration.createTestRun();
    await AzureIntegration.updateTestResults(outcome);


  }
}
